@@ -1,7 +1,18 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
+# -*- coding:utf-8 -*-
+# Copyright 2019 Huawei Technologies Co.,Ltd.
+# Licensed under the Apache License, Version 2.0 (the "License"); you may not use
+# this file except in compliance with the License.  You may obtain a copy of the
+# License at
+
+# http://www.apache.org/licenses/LICENSE-2.0
+
+# Unless required by applicable law or agreed to in writing, software distributed
+# under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
+# CONDITIONS OF ANY KIND, either express or implied.  See the License for the
+# specific language governing permissions and limitations under the License.
+
 import weakref, collections
-from obs import const
 import time
 
 
@@ -59,5 +70,4 @@ class _LocalCacheProcess(object):
                 pass
         
         
-# LocalCache = _LocalCacheThread if const.IS_WINDOWS else _LocalCacheProcess
 LocalCache = _LocalCacheThread
\ No newline at end of file
